#Email Demo for the Terminus Terminal
# Janus Model#  GSM864QP V1.0
#
#Copyright  2008, Janus Remote Communications
#All rights reserved.
#
#Redistribution and use in source and binary forms, with or without
#modification, are permitted provided that the following conditions
#are met:
#
#Redistributions of source code must retain the above copyright notice,
#this list of conditions and the following disclaimer.
#
#Redistributions in binary form must reproduce the above copyright
#notice, this list of conditions and the following disclaimer in
#the documentation and/or other materials provided with the distribution.
#
#
#THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS``AS
#IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
#TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
#PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR
#CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
#EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
#PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import MDM              #code owned by: Telit
import MOD              #code owned by: Telit
import timers           #code owned by: Telit
import sys              #code owned by: Telit

import ATC              #code owned by: Janus Remote Communications
import GSM864QP_SER     #code owned by: Janus Remote Communications
import DEBUG            #code owned by: Janus Remote Communications

##------------------------------------------------------------------------------------------------------------------
## Release Information:
##  V1.0 (Thomas W. Heck, 11/12/2008)   :   Initial release
##------------------------------------------------------------------------------------------------------------------

##------------------------------------------------------------------------------------------------------------------
## NOTES
##  1.)  exceptions.pyo must be loaded on the Telit module in order for this code to work correctly
##  2.)  Don't include the following line of code in any of your modules: 'import exceptions'
##  3.)  See Main Script, Application Specific Configuration section.  Demo must be altered for customer settings
##  4.)  print statements are displayed when using the IDE to debug application
##  5.)  Telit module can't logon to SMTP servers that require TLS or SSL encryption
##  6.)  If you need to send emails to users outside your network you will need to connect to an open relay SMTP server or non-secure STMP server
##  7.)  Please read "Terminus Email Demonstration Guide.pdf"
##------------------------------------------------------------------------------------------------------------------

class myApp:
    BAND = ''
    NETWORK = ''
    GPRS_USERID = ''
    GPRS_PASSWORD = ''
    APN = ''
    IP = ''
    PORT = ''
    PROTOCOL = ''
    SMS = ''
    INTERVAL = ''
    EMAIL_TO_ADDRESS = ''
    EMAIL_FROM_ADDRESS = ''
    SMTP_SERVER_ADDRESS = ''
    SMTP_USERID = ''
    SMTP_PASSWORD = ''

#Main Script
# ###############################################################
try:

    ##-----------------------------------------------------------------------------------------------------
    ## Application Specific Configuration
    ##-----------------------------------------------------------------------------------------------------

    ## BAND
    ## Please refer to AT Command guide for AT#BND
    ## If Terminal used in North America (BAND = '3')
    myApp.BAND = '3'

    ## NETWORK
    ## If Terminal used on ATT / Cingular in North America (NETWORK = 'ATT')
    ## Else (NETWORK = 'GSM')
    myApp.NETWORK = 'ATT'

    ## GPRS_USERID
    ## USERID for GPRS connection - ask your network operator for setting
    ## If GPRS_USERID is not required leave empty 
    myApp.GPRS_USERID   = ''

    ## GPRS_PASSWORD
    ## PASSWORD for GPRS connection - ask your network operator for setting
    ## If GPRS_PASSWORD is not required leave empty 
    myApp.GPRS_PASSWORD    = ''

    ## APN
    ## Gateway Address for GPRS traffic
    ## This setting is GSM Provider and possible customer specific when a VPN is used
    ## This demo is defaulted with proxy that is used for ATT wireless settings from pre-Cingular days
    ## You MUST obtain the APN setting for your GSM account.  Please call GSM provider!  Janus can't help you with this.
    myApp.APN = 'proxy'

    ## IP
    ## IP address of server on the Internet which Terminus will connect to send GPS information
    ## Address in this example is not operational for customer evaluation.  Customer must have their own server
    ## setup to interact with this demo.
    myApp.IP = 'xxx.xxx.xxx.xxx'   ##Not used in this demo

    ## PORT
    ## PORT number of server on the Internet which Terminus will connect to send GPS information
    ## PORT number in this example is not operational for customer evaluation.  Customer must have their own server
    ## setup to interact with this demo.
    myApp.PORT = '5556'   ##Not used in this demo

    ## PROTOCOL
    ## If customer is using TCPIP (PROTOCOL = 'TCPIP')
    ## Else leave blank (PROTOCOL = '')
    myApp.PROTOCOL = 'TCPIP'   ##Not used in this demo

    ## SMS
    ## SMS Designation phone number (SMS = '+16305551212')
    myApp.SMS = '+16305551212'  ##Not used in this demo

    ## EMAIL_TO_ADDRESS
    ## Email address (To:)
    myApp.EMAIL_TO_ADDRESS = 'john.doe@gmail.com'

    ## EMAIL_FROM_ADDRESS
    ## Email address (From:)
    ## Unless SMTP server permits relay, this must match with MAIL_USER and MAIL_PASSW.
    myApp.EMAIL_FROM_ADDRESS = 'the.terminus@yourURL.com'

    ## SMTP_SERVER_ADDRESS
    ## The URL or IP address of the Mail eXchange(MX) SMTP server
    ## The following URL was looked up for gmail.com
    ## Use nslookup in Linux or Windows to find your networks MX server
    ## c:\nslookup.exe
    ## > SET TYPE=MX
    ## > gmail.com
    ##
    ## Helpful Links:
    ##      http://en.wikipedia.org/wiki/Simple_Mail_Transfer_Protocol
    ##      http://en.wikipedia.org/wiki/Mx_record
    myApp.SMTP_SERVER_ADDRESS   = 'gmail-smtp-in.l.google.com'

    ## SMTP_USERID
    ## Logon username for the SMTP server
    myApp.SMTP_USERID     = ''

    ## SMTP_PASSWORD
    ## Logon password for the SMTP server
    myApp.SMTP_PASSWORD     = ''

    ## INTERVAL
    ## How many seconds the demo will wait before sending an email to the SMTP server.
    ## In this demo, an email will be sent every 60 seconds (INTERVAL = '60')
    myApp.INTERVAL = '60'    

    MOD.watchdogEnable(300)

    # The following code is used to determine if the code is running in the IDE or the Terminus
    try:
        
        RUN_MODE = 0
        test = float(RUN_MODE)  #Running in IDE if this command executes with no exception
    except:
        #This branch is executed if Script is running in Telit Module
        #Telit doesn't support type float
        RUN_MODE = 1
        GSM864QP_SER.init_parameters ('9600','8N1')

    DEBUG.CLS(RUN_MODE)   #Clear screen command for VT100 terminals
    DEBUG.sendMsg("Janus Terminus Email Demo Script has Started\r\n",RUN_MODE)  
  
    #Apply Network Specific settings see myApp.xxxx assignment above
    if (myApp.NETWORK == "ATT"):
        #Set module to work on US ATT Network  
        res = ATC.sendAtCmd("AT#ENS?",ATC.properties.CMD_TERMINATOR,3,2)        #query ENS setting
        if (res == "#ENS: 0"):
            res = ATC.sendAtCmd('AT#ENS=1',ATC.properties.CMD_TERMINATOR,3,2)   #sets all ATT requirements
            MOD.sleep(15)                                                       #required to halt Python thread and allow NVM Flash to update
            res = ATC.sendAtCmd('AT#REBOOT',ATC.properties.CMD_TERMINATOR,3,2)  #must reboot to take effect
        res = ATC.sendAtCmd("AT#SELINT?",ATC.properties.CMD_TERMINATOR,3,2)     #query SELINT setting
        if (res != "#SELINT: 2"):
            res = ATC.sendAtCmd('AT#SELINT=2',ATC.properties.CMD_TERMINATOR,3,2)#use of most recent AT command set
            MOD.sleep(15)                                                       #required to halt Python thread and allow NVM Flash to update
            res = ATC.sendAtCmd('AT#REBOOT',ATC.properties.CMD_TERMINATOR,3,2)  #must reboot to take effect
        
    else:
        #Set module to work on all other Networks
        res = ATC.sendAtCmd('AT#ENS?',ATC.properties.CMD_TERMINATOR,3,2) 
        if (res == "#ENS: 1"):
            res = ATC.sendAtCmd('AT#ENS=0',ATC.properties.CMD_TERMINATOR,3,2)   #disable ATT requirements
            MOD.sleep(15)                                                       #required to halt Python thread and allow NVM Flash to update
            res = ATC.sendAtCmd('AT#REBOOT',ATC.properties.CMD_TERMINATOR,3,2)  #must reboot to take effect
        res = ATC.sendAtCmd("AT#SELINT?",ATC.properties.CMD_TERMINATOR,3,2)     #query SELINT setting
        if (res != "#SELINT: 2"):
            res = ATC.sendAtCmd('AT#SELINT=2',ATC.properties.CMD_TERMINATOR,3,2)#use of most recent AT command set
            MOD.sleep(15)                                                       #required to halt Python thread and allow NVM Flash to update
            res = ATC.sendAtCmd('AT#REBOOT',ATC.properties.CMD_TERMINATOR,3,2)  #must reboot to take effect
        res = ATC.sendAtCmd('AT#STIA=2,10',ATC.properties.CMD_TERMINATOR,3,2)   #enable SAT - SIM Application Tool-Kit
        res = ATC.sendAtCmd('AT#BND='+myApp.BAND,ATC.properties.CMD_TERMINATOR,3,2)       #set bands to 850/1900
        res = ATC.sendAtCmd('AT#AUTOBND=1',ATC.properties.CMD_TERMINATOR,3,2)	#enable Quad band system selection
        res = ATC.sendAtCmd('AT#PLMNMODE=1',ATC.properties.CMD_TERMINATOR,3,2)	#enable EONS (enhanced operator naming scheme) 

    DEBUG.sendMsg("Connecting to GSM network",RUN_MODE)

    #Wait until registered to GSM Network
    exitLoop = 1
    while (exitLoop == 1):
        res = ATC.sendAtCmd('AT+CREG?',ATC.properties.CMD_TERMINATOR,0,5)
        DEBUG.sendMsg(".",RUN_MODE)
        if (res[res.rfind(',')+1:len(res)] == '5' or res[res.rfind(',')+1:len(res)] == '1'):
            exitLoop = 0

    DEBUG.sendMsg("\r\nConnected to GSM network\r\n",RUN_MODE)

    MOD.watchdogReset()

    #Init GPRS Settings
    #Pass in: PDP Index and network specific APN
    ATC.initGPRS('1',myApp.APN,myApp.GPRS_USERID,myApp.GPRS_PASSWORD)

    ###################################################################################################################################
    ## Define E-mail settings
    ##################################################################################################################################

    #Authentication User ID
    res = ATC.sendAtCmd('AT#USERID="' + myApp.GPRS_USERID + '"',ATC.properties.CMD_TERMINATOR,3,2)

    #Authentication Password
    res = ATC.sendAtCmd('AT#PASSW="' + myApp.GPRS_PASSWORD + '"',ATC.properties.CMD_TERMINATOR,3,2)

    #E-mail SMTP Server
    res = ATC.sendAtCmd('AT#ESMTP="' + myApp.SMTP_SERVER_ADDRESS + '"',ATC.properties.CMD_TERMINATOR,3,2)

    #E-mail Authentication User Name
    res = ATC.sendAtCmd('AT#EUSER="' + myApp.SMTP_USERID + '"',ATC.properties.CMD_TERMINATOR,3,2)

    #E-mail Authentication Password
    res = ATC.sendAtCmd('AT#EPASSW="' + myApp.SMTP_PASSWORD + '"',ATC.properties.CMD_TERMINATOR,3,2)

    #E-mail Sender Address
    res = ATC.sendAtCmd('AT#EADDR="' + myApp.EMAIL_FROM_ADDRESS + '"',ATC.properties.CMD_TERMINATOR,3,2)

    ## E-mail settings Complete
    ###################################################################################################################################

    # Start timeout timer            
    timerB = timers.timer(0)
    timerB.start(1) 

    # Loop forever, without this loop the script would run once and exit script mode.  On reboot or power-on the script would run once more
    while (1):
                
        exitCode = -1
        while (exitCode==-1):

            MOD.watchdogReset()

            #If interval timer expires then send packet to server       
            if (timerB.isexpired()):

                DEBUG.sendMsg("Sending Email\r\n",RUN_MODE)                            

                res = ATC.sendEMAIL(myApp.EMAIL_TO_ADDRESS,'Terminus Email Demo','Customer,\r\n\nThis message was sent by the Terminus device!\r\n\n Sincerely,\r\n\n Janus Application Group',ATC.properties.CMD_TERMINATOR,myApp.GPRS_USERID,myApp.GPRS_PASSWORD,3,180)

                if (res == 0):
                    DEBUG.sendMsg("Email has been sent\r\n",RUN_MODE)
                else:
                    DEBUG.sendMsg("Email failed to send correctly\r\n",RUN_MODE)

                exitCode = 0

        DEBUG.sendMsg("Waiting for INTERVAL to expire\r\n",RUN_MODE)

        ## Re-Start timeout timer            
        timerB = timers.timer(0)
        timerB.start(int(myApp.INTERVAL))

except:
    DEBUG.sendMsg('Script encountered an exception\r\n',RUN_MODE)
    DEBUG.sendMsg('Exception Type: ' + str(sys.exc_type) + "\r\n",RUN_MODE)
    DEBUG.sendMsg('MODULE -> TerminusEMAIL\r\n',RUN_MODE)
    MOD.sleep(15)                                                       #required to halt Python thread and allow NVM Flash to update
    res = ATC.sendAtCmd('AT#REBOOT',ATC.properties.CMD_TERMINATOR,0,5)  #must reboot or Script will not restart until a power cycle occurs

      